local ArcanumGuildHall = _G['ArcanumGuildHall']

local res = ArcanumGuildHallMediaRes
local LAM2 = LibAddonMenu2

function ArcanumGuildHall:GetGuildHousesFromDescription()
    local entries = {}
    local guildDesc = GetGuildDescription(self.guildId)
    for title, houseId, name in string.gmatch(guildDesc, "(%S+%s)%|H1:housing:(%d+):(%S+)%|h%|h") do
        if (#entries > 0) then
            table.insert(entries, { label = "-" })
        end
        table.insert(entries, { label = title, callback = function() self:PortToHouse(name, houseId) end, })
    end

    return entries
end

function ArcanumGuildHall:CreateMenuEntries()
    local entries2 = {
        {
            label = res.IconGrpShare .. ArcanumGuildHall.GetDefaultLocaleString("SHARE_ALLDAILIES"),
            callback = function() self:ShareAllDailies() end,
        },
        {
            label = res.IconGrpShare .. ArcanumGuildHall.GetDefaultLocaleString("SHARE_ZONEDAILIES"),
            callback = function() self:ShareZoneDailies() end,
        },
        {
            label = res.IconGrpLead .. ArcanumGuildHall.GetDefaultLocaleString("PORT_GROUPLEADER"),
            callback = function() JumpToGroupLeader() end,
        },
        {
            label = "-"
        },
        {
            label = res.IconGrpLeave .. GetString(SI_GROUP_LEAVE),
            callback = function() GroupLeave() end,
        }
    }

    return self:GetGuildHousesFromDescription(), entries2, self:CreateTeleportEntryList(), self:CreateTeleportNodeEntryList()
end

function ArcanumGuildHall:ShowMenuLeft()
    local entries1, entries2, entries3, entries4 = self:CreateMenuEntries()

    ClearMenu()
    AddCustomSubMenuItem("" .. res.IconGHouse .. ArcanumGuildHall.GetDefaultLocaleString("GUILD_HOUSES"), entries1, normalColor)

    if (IsUnitGrouped("player")) then
        AddCustomMenuItem("-", function()
        end)
        AddCustomSubMenuItem("" .. res.IconGrpTool .. ArcanumGuildHall.GetDefaultLocaleString("BUTTON_GROUPTOOL"), entries2, normalColor)
    end

    AddCustomMenuItem("-", function()
    end)
    AddCustomSubMenuItem("" .. res.IconWay .. ArcanumGuildHall.GetDefaultLocaleString("PORT_BUTTON_TELEPORT"), entries3, normalColor)
    AddCustomSubMenuItem("" .. res.IconWay .. ArcanumGuildHall.GetDefaultLocaleString("PORT_BUTTON_TELEPORTWAY") .. " " .. GetRecallCost(node) .. res.IconGold, entries4, normalColor)
    AddCustomMenuItem(res.IconGMHouse .. GetString(SI_SOCIAL_MENU_VISIT_HOUSE), function()
        RequestJumpToHouse(GetHousingPrimaryHouse())
    end)
    ShowMenu()
    AddCustomMenuItem("-", function()
    end)
    AddCustomMenuItem("" .. res.IconPld .. ArcanumGuildHall.GetDefaultLocaleString("PLEDGES_BUTTON"), function()
        self:ListPledges()
    end)
    AddCustomMenuItem("-", function()
    end)
    AddCustomMenuItem(res.IconDiscord .. ArcanumGuildHall.GetDefaultLocaleString("OUR_DISCORD"), function()
        RequestOpenUnsafeURL("https://discord.gg/3FngWCQ")
    end)
    AddCustomMenuItem(res.IconWeb .. ArcanumGuildHall.GetDefaultLocaleString("OUR_WEBSITE"), function()
        RequestOpenUnsafeURL("https://arcanumartis.de")
    end)
    AddCustomMenuItem(res.IconOpt .. GetString(SI_GAME_MENU_SETTINGS), function()
        LAM2:OpenToPanel(self.panel)
    end)
    AddCustomMenuItem("-", function()
    end)
    AddCustomMenuItem(res.IconRldUI .. "ReloadUI", function()
        ReloadUI()
    end)
    ShowMenu()
end

function ArcanumGuildHall:ShowMenuRight()
    ClearMenu()
    AddCustomMenuItem(res.IconRnR .. ArcanumGuildHall.GetDefaultLocaleString("BINDING_REPAIR_RECHARGE"), function()
        ArcanumGuildHall.RepairRecharge()
    end)
    AddCustomMenuItem(res.IconRepair .. ArcanumGuildHall.GetDefaultLocaleString("BINDING_REPAIR_WITH_KITS"), function()
        ArcanumGuildHall.RepairItemsWithKits()
    end)
    AddCustomMenuItem(res.IconRecharge .. ArcanumGuildHall.GetDefaultLocaleString("BINDING_RECHARGE_WITH_GEMS"), function()
        ArcanumGuildHall.RechargeItemsWithGems()
    end)
    ShowMenu()
end

function ArcanumGuildHall:InitializeChatIcon()
    local ptoGHall = WINDOW_MANAGER:CreateControl("ArcanumGuildHall1", ZO_ChatWindow, CT_BUTTON)
    ptoGHall:SetDimensions(20, 20)
    ptoGHall:SetAnchor(TOPLEFT, ZO_ChatOptionsSectionLabel, TOPRIGHT, -100, 11)
    ptoGHall:SetHandler("OnMouseEnter", function(control)
        InitializeTooltip(InformationTooltip, control)
        InformationTooltip:AddLine("" .. res.IconAA .. " " .. "|cffffffArcanum Artis|r" .. " " .. res.IconAA .. "")
        InformationTooltip:AddVerticalPadding(-15)
        InformationTooltip:AddLine("" .. res.IconDivider .. "")
        InformationTooltip:AddVerticalPadding(-10)
        InformationTooltip:AddLine("" .. res.IconLeftClk .. ArcanumGuildHall.GetDefaultLocaleString("MENU_LEFT") .. res.IconRightClk .. ArcanumGuildHall.GetDefaultLocaleString("MENU_RIGHT"))
    end)
    ptoGHall:SetHandler("OnMouseExit", function(_)
        ClearTooltip(InformationTooltip)
    end)

    ptoGHall:SetHandler("OnMouseUp", function(control, button, upInside)
        if button == MOUSE_BUTTON_INDEX_LEFT then
            self:ShowMenuLeft()
        elseif button == MOUSE_BUTTON_INDEX_RIGHT then
            self:ShowMenuRight()
        end
    end)
    self.chatIcon = ptoGHall
end

function ArcanumGuildHall:ShowChatIcon(show)
    self.chatIcon:SetHidden(not show)
end

function ArcanumGuildHall:SetChatIconTexture(monochrome)
    self.chatIcon:SetNormalTexture(monochrome and "ArcanumGuildHall/imgs/aaguild_mono.dds" or "ArcanumGuildHall/imgs/aaguild.dds")
    self.chatIcon:SetPressedTexture(monochrome and "ArcanumGuildHall/imgs/aaguild_mono_pressed.dds" or "ArcanumGuildHall/imgs/aaguild_pressed.dds")
    self.chatIcon:SetMouseOverTexture("ArcanumGuildHall/imgs/aaguild_over.dds")
end
