local ArcanumGuildHall = _G['ArcanumGuildHall']

local res = ArcanumGuildHallMediaRes
local LCM = LibCustomMenu

local msg = ZO_ChatSystem_GetEventHandlers()[EVENT_CHAT_MESSAGE_CHANNEL]
local NOTE_PATTERN_HELPER = "|cAAHELP|r"
local GroupNames = {}

function ArcanumGuildHall:UpdateGroupMembers()
    local size = GetGroupSize()
    for i = 1, size do
        GroupNames[GetUnitName('group' .. i)] = GetUnitDisplayName('group' .. i)
    end
end

local function getAccountName(name)
    if (zo_strsub(name, 1, 1) == "@") then
        return name
    elseif IsPlayerInGroup(name) then
        return GroupNames[name]
    else
        return nil
    end
end

function ArcanumGuildHall:IsGuildMember(accountName)
    -- Check Arcanum
    local arcanumIndex = GetGuildMemberIndexFromDisplayName(self.guildId, accountName)
    if arcanumIndex ~= nil then
        return true, true
    end

    -- Check all guilds
    for index = 1, GetNumGuilds() do
        local guildId = GetGuildId(index)
        local memberIndex = GetGuildMemberIndexFromDisplayName(guildId, accountName)
        if memberIndex ~= nil then
            return true, false
        end
    end
    return false, false
end

function ArcanumGuildHall:InitializeChatMenu()

    LCM:RegisterPlayerContextMenu(function(playerName, rawName)
        local accountName = getAccountName(playerName)
        if (accountName == nil) then
            return
        end

        local isGuildMember, isArcanumMember = self:IsGuildMember(accountName)

        -- Fallback for special cases where chat account name and guild account name are different
        if (not isGuildMember and not IsPlayerInGroup(accountName)) then
            local fallbackName = getAccountName("@"..zo_strformat("<<c:1>>", rawName))
            local fallbackMember, fallbackArcanum = self:IsGuildMember(fallbackName)
            if (fallbackMember) then
                accountName, isGuildMember, isArcanumMember = fallbackName, fallbackMember, fallbackArcanum
            end
        end

        -- Arcanum Rank info
        if isArcanumMember then
            local arcanumIndex = GetGuildMemberIndexFromDisplayName(self.guildId, accountName)
            local _, _, rankIndex = GetGuildMemberInfo(self.guildId, arcanumIndex)
            local rankName = GetGuildRankCustomName(self.guildId, rankIndex)

            AddCustomMenuItem("-", function() end)
            AddCustomMenuItem(res.IconAA .. res.Ccolor7 .. " " .. accountName .. ":" .. " " .. rankName .. "|r", function() end)
            AddCustomMenuItem("-", function() end)
        end

        -- Arcanum Guild Invite
        if not isArcanumMember and DoesPlayerHaveGuildPermission(self.guildId, GUILD_PERMISSION_INVITE) == true then
            AddCustomMenuItem(ArcanumGuildHall.GetDefaultLocaleString("CHAT_NEW_INVITE") .. " " .. res.Ccolor8 .. GetGuildName(self.guildId) .. "|r", function()
                GuildInvite(self.guildId, accountName)
            end)
        end

        -- New Mail
        AddCustomMenuItem(ArcanumGuildHall.GetDefaultLocaleString("CHAT_NEW_MESSAGE"), function()
            if MAIL_SEND:IsHidden() then
                MAIL_SEND:ComposeMailTo(accountName)
            else
                MAIL_SEND:SetReply(accountName)
            end
        end)

        -- Teleport
        if IsPlayerInGroup(accountName) then
            AddCustomMenuItem(GetString(SI_SOCIAL_MENU_JUMP_TO_PLAYER), function()
                JumpToGroupMember(accountName)
            end)
        elseif IsFriend(accountName) then
            AddCustomMenuItem(GetString(SI_SOCIAL_MENU_JUMP_TO_PLAYER), function()
                JumpToFriend(accountName)
            end)
        elseif isGuildMember then
            AddCustomMenuItem(GetString(SI_SOCIAL_MENU_JUMP_TO_PLAYER), function()
                JumpToGuildMember(accountName)
            end)
        end

        -- Teleport to main house
        AddCustomMenuItem(GetString(SI_SOCIAL_MENU_VISIT_HOUSE), function()
            JumpToHouse(accountName)
        end)
    end, LCM.CATEGORY_LATE)
end

function ArcanumGuildHall:findGuildIndex()
    for i = 1, 5 do
        if GetGuildId(i) == ArcanumGuildHall.guildId then
            return i
        end
    end
    return 0
end

function ArcanumGuildHall:getTime()
    if self.db.showChatTimestamp then
        return "[" .. self:GetTimeString() .. "] "
    else
        return ""
    end
end

function ArcanumGuildHall:isGuildChannel(channelType)
    local index = self.guildIndex
    if index == 0 then
        return
    end

    if channelType == _G["CHAT_CHANNEL_GUILD_" .. index] or channelType == _G["CHAT_CHANNEL_OFFICER_" .. index] then
        return true
    end
    return false
end

function ArcanumGuildHall:SetTextColor(channelType, fromDisplay)
    if self.db.showChatGuildColor == 0 or not self:isGuildChannel(channelType) then
        return ""
    end

    local memberIndex = GetGuildMemberIndexFromDisplayName(self.guildId, fromDisplay)
    local _, _, rankIndex = GetGuildMemberInfo(self.guildId, memberIndex)

    local rankName = GetGuildRankCustomName(self.guildId, rankIndex) or ""
    local rankColor = rankName:sub(3, 8)

    return rankColor
end

function ArcanumGuildHall:GetIcon(channelType, fromDisplay)
    if self.db.showChatGuildIcon == 0 or not self:isGuildChannel(channelType) then
        return ""
    end

    local memberIndex = GetGuildMemberIndexFromDisplayName(self.guildId, fromDisplay)
    local _, memberNote, rankIndex = GetGuildMemberInfo(self.guildId, memberIndex)
    local iconIndex = GetGuildRankIconIndex(self.guildId, rankIndex)
    local iconColor = ArcanumGuildHall:SetTextColor(channelType, fromDisplay) .. "%s"

    if self.db.showChatGuildIcon == 2 and PlainStringFind(memberNote, NOTE_PATTERN_HELPER) == false and rankIndex >= 6 then
        return ""
    end

    local icon
    if PlainStringFind(memberNote, NOTE_PATTERN_HELPER) == true then
        icon = res.IconHlp
    else
        icon = GetGuildRankLargeIcon(iconIndex)
    end

    local iconSize = ZO_ChatSystem:GetFontSizeFromSetting() * 1.5
    local setIcon

    if self.db.showChatGuildIconColor == false then
        setIcon = zo_iconFormatInheritColor(icon, iconSize, iconSize)
    else
        setIcon = string.format("|c" .. iconColor .. "|r", zo_iconFormatInheritColor(icon, iconSize, iconSize))
    end

    return setIcon
end

local function handleChatMessage(channelType, fromName, messageText, isCustomerService, fromDisplay)
    return ArcanumGuildHall:getTime() .. ArcanumGuildHall:GetIcon(channelType, fromDisplay) .. msg(channelType, fromName, messageText, isCustomerService, fromDisplay)
end

function ArcanumGuildHall:ShowChatTimestamp()
    ZO_ChatSystem_AddEventHandler(EVENT_CHAT_MESSAGE_CHANNEL, handleChatMessage)
end
