local ArcanumGuildHall = _G['ArcanumGuildHall']

local LN = LibNotifications
local LN_provider = LN:CreateProvider()
local shown = false

local function removeNotification(notificationId)
    local provider = LN_provider
    table.remove(provider.notifications, notificationId)
    provider:UpdateNotifications()
end

local function acceptNotification(data)
    ArcanumGuildHall.db.showReminder = false
    removeNotification(data.notificationId)
end

local function declineNotification(data)
    ArcanumGuildHall.db.showReminder = true
    removeNotification(data.notificationId)
end

local function addNotification()
    if shown then
        return

    end
    shown = true
    local msg = {
        dataType = NOTIFICATIONS_REQUEST_DATA,
        secsSinceRequest = ZO_NormalizeSecondsSince(0),
        message = ArcanumGuildHall.db.defaultNotMessage,
        heading = ArcanumGuildHall.db.defaultNotTitle,
        texture = "ArcanumGuildHall/imgs/aaguild.dds",
        shortDisplayText = "!",
        controlsOwnSounds = false,
        keyboardAcceptCallback = function(data)
            acceptNotification(data)
        end,
        keyboardDeclineCallback = function(data)
            declineNotification(data)
        end,
        gamepadAcceptCallback = function(data)
            acceptNotification(data)
        end,
        gamepadDeclineCallback = function(data)
            declineNotification(data)
        end,
    }
    table.insert(LN_provider.notifications, msg)
    LN_provider:UpdateNotifications()
end

function ArcanumGuildHall:ShowFirstTimeInfo()
    if self.db.firstTimeInfoShown then
        return
    end

    self.db.firstTimeInfoShown = true
    local msg = {
        dataType = NOTIFICATIONS_REQUEST_DATA,
        secsSinceRequest = ZO_NormalizeSecondsSince(0),
        message = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_NOTIFICATION_MESSAGE_FIRST"),
        heading = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_NOTIFICATION_TITLE"),
        texture = "ArcanumGuildHall/imgs/aaguild.dds",
        shortDisplayText = "!",
        controlsOwnSounds = false,
        keyboardAcceptCallback = function(data)
            self.db.enableReminder = true
            self:ShowNotification()
            removeNotification(data.notificationId)
        end,
        keyboardDeclineCallback = function(data)
            self.db.enableReminder = false
            removeNotification(data.notificationId)
        end,
        gamepadAcceptCallback = function(data)
            self.db.enableReminder = true
            self:ShowNotification()
            removeNotification(data.notificationId)
        end,
        gamepadDeclineCallback = function(data)
            self.db.enableReminder = false
            removeNotification(data.notificationId)
        end,
    }
    table.insert(LN_provider.notifications, msg)
    LN_provider:UpdateNotifications()
end

function ArcanumGuildHall:ShowNotification()
    if not self.db.enableReminder then
        return
    end

    local currentDay = os.date("%A")
    if self.db.selectedReminderDay ~= currentDay then
        self.db.showReminder = true
    else
        if self.db.showReminder then
            shown = false
            EVENT_MANAGER:RegisterForUpdate(self.addonName, 1, addNotification)
        end
    end
end
