local ArcanumGuildHall = _G['ArcanumGuildHall']

local res = ArcanumGuildHallMediaRes
local data = ArcanumGuildHallTeleportData

local function cutStringAtLetter(text, letter)
    if text ~= nil then
        local pos = string.find(text, letter, nil, true)

        if pos then
            text = string.sub(text, 1, pos - 1)
        end
    end

    return text
end

local function isCollectibleUnlocked(collectibleId)
    if (collectibleId == nil) then
        return true
    end

    local _, _, _, _, unlocked = GetCollectibleInfo(collectibleId)
    return unlocked
end

local function getZoneLabel(zone)
    if (zone and zone.name and zone.label) then
        return zone.label .. zone.name .. "|r"
    elseif zone.label then
        return zone.label
    else
        return ""
    end
end

function ArcanumGuildHall:CreateTeleportEntryList()
    local entryList = {}
    for _, zone in pairs(data.TeleportList) do

        if (isCollectibleUnlocked(zone.collectibleId)) then
            entryList[#entryList + 1] = {
                label = getZoneLabel(zone),
                callback = function()
                    ArcanumGuildHall:Teleport(zone)
                end,
            }
        end
    end
    return entryList
end

function ArcanumGuildHall:CreateTeleportNodeEntryList()
    local entryList = {}
    for _, node in pairs(data.TeleportNodeList) do

        if (isCollectibleUnlocked(node.collectibleId) and HasCompletedFastTravelNodePOI(node.id)) then
            entryList[#entryList + 1] = {
                label = getZoneLabel(node),
                callback = function()
                    ArcanumGuildHall:CityTeleport(node)
                end,
            }
        end
    end
    return entryList
end

function ArcanumGuildHall:Teleport(zone)
    local porting = false

    if IsInAvAZone() then
        CHAT_ROUTER:AddSystemMessage(res.IconAA .. res.Ccolor3 .. " " .. GetString(SI_TOOLTIP_WAYSHRINE_CANT_RECALL_AVA) .. "|r")
    else

        for p = 1, GetGroupSize() do
            local pTag = GetGroupUnitTagByIndex(p)
            if GetUnitZoneIndex(pTag) == zone.id and CanJumpToGroupMember(pTag) and porting == false and GetUnitDisplayName(pTag) ~= GetUnitDisplayName("player") then
                JumpToGroupMember(GetUnitName(pTag))
                CHAT_ROUTER:AddSystemMessage(res.IconAA .. res.Ccolor1 .. " " .. GetUnitDisplayName(pTag) .. ArcanumGuildHall.GetDefaultLocaleString("PORT_CHATTEXT") .. zone.name .. "|r")
                porting = true
                break
            end
        end

        if porting == false then
            for f = 1, GetNumFriends() do
                local CharInfo = {}
                CharInfo.displayName, CharInfo.Note, CharInfo.status, CharInfo.secsSinceLogoff = GetFriendInfo(f)
                CharInfo.hasCharacter, CharInfo.characterName, CharInfo.zoneName, CharInfo.classType, CharInfo.alliance, CharInfo.level, CharInfo.championRank, CharInfo.zoneId = GetFriendCharacterInfo(f)
                if CharInfo.status ~= PLAYER_STATUS_OFFLINE and GetZoneIndex(CharInfo.zoneId) == zone.id and porting == false and CharInfo.displayName ~= GetUnitDisplayName("player") then
                    JumpToFriend(CharInfo.displayName)
                    CHAT_ROUTER:AddSystemMessage(res.IconAA .. res.Ccolor1 .. " " .. ArcanumGuildHall.GetDefaultLocaleString("PORT_CHATTEXT") .. zone.name .. "|r")
                    porting = true
                    break
                end
            end
        end

        if porting == false then
            for g = 1, GetNumGuilds() do
                for m = 1, GetNumGuildMembers(GetGuildId(g)) do
                    local CharInfo = {}
                    CharInfo.displayName, CharInfo.Note, CharInfo.GuildMemberRankIndex, CharInfo.status, CharInfo.secsSinceLogoff = GetGuildMemberInfo(GetGuildId(g), m)
                    CharInfo.hasCharacter, CharInfo.characterName, CharInfo.zoneName, CharInfo.classType, CharInfo.alliance, CharInfo.level, CharInfo.championRank, CharInfo.zoneId = GetGuildMemberCharacterInfo(GetGuildId(g), m)
                    CharInfo.guildIndex = g
                    if CharInfo.status ~= PLAYER_STATUS_OFFLINE and GetZoneIndex(CharInfo.zoneId) == zone.id and porting == false and CharInfo.displayName ~= GetUnitDisplayName("player") then
                        JumpToGuildMember(CharInfo.displayName)
                        CHAT_ROUTER:AddSystemMessage(res.IconAA .. res.Ccolor1 .. " " .. ArcanumGuildHall.GetDefaultLocaleString("PORT_CHATTEXT") .. zone.name .. "|r")
                        porting = true
                        break
                    end
                end
            end
        end

        if porting == false then
            CHAT_ROUTER:AddSystemMessage(res.IconAA .. res.Ccolor3 .. " " .. ArcanumGuildHall.GetDefaultLocaleString("PORT_CHATTEXT_FALSE") .. "|r")
        end
    end
end

function ArcanumGuildHall:CityTeleport(node)
    local porting = false

    if IsInAvAZone() then
        CHAT_ROUTER:AddSystemMessage(res.IconAA .. res.Ccolor3 .. " " .. GetString(SI_TOOLTIP_WAYSHRINE_CANT_RECALL_AVA) .. "|r")
    else

        if HasCompletedFastTravelNodePOI(node.id) then
            FastTravelToNode(node.id)
            CHAT_ROUTER:AddSystemMessage(res.IconAA .. res.Ccolor1 .. " " .. ArcanumGuildHall.GetDefaultLocaleString("PORT_CHATTEXT_NODE") .. node.name .. " " .. ArcanumGuildHall.GetDefaultLocaleString("PORT_CHATTEXT_NODE_COST") .. GetRecallCost(node.id) .. res.IconGold .. "|r")
            porting = true

        end

        if not HasCompletedFastTravelNodePOI(node.id) then
            CHAT_ROUTER:AddSystemMessage(res.IconAA .. res.Ccolor3 .. " " .. ArcanumGuildHall.GetDefaultLocaleString("PORT_CHATTEXT_NODE_FALSE") .. "|r")
        end
    end
end

function ArcanumGuildHall:PortToHouse(name, houseId)
    if IsInAvAZone() then
        CHAT_ROUTER:AddSystemMessage(res.IconAA .. res.Ccolor3 .. " " .. GetString(SI_TOOLTIP_WAYSHRINE_CANT_RECALL_AVA) .. "|r")

    else

        if (GetDisplayName() == name) then
            RequestJumpToHouse(houseId)
        else
            JumpToSpecificHouse(name, houseId)
        end
    end
end
